local L = LibStub("AceLocale-3.0"):GetLocale("ProximoGrid", true)
local media = LibStub("LibSharedMedia-3.0");
-- BORDER
ProximoGridIndicatorBorder = ProximoGridIndicator.NewModule("border", L["Border"], {}, {
	"statusPlayerTarget",
});
ProximoGridIndicatorBorder.options = {};
ProximoGridIndicatorBorder.borderWidth = 1;

function ProximoGridIndicatorBorder:CreateIndicator(frame)
	local borderWidth = ProximoGridIndicatorBorder.borderWidth;
	frame:SetBackdrop({
		bgFile = "Interface\\Addons\\ProximoGrid\\white16x16", tile = true, tileSize = 16,
		edgeFile = "Interface\\Addons\\ProximoGrid\\white16x16", edgeSize = borderWidth,
		insets = {left = borderWidth, right = borderWidth, top = borderWidth, bottom = borderWidth},
	})
	frame:SetBackdropBorderColor(0,0,0,1)
	frame:SetBackdropColor(0,0,0,1)
end

function ProximoGridIndicatorBorder:ResetIndicator(frame)
	frame:SetBackdropBorderColor(0, 0, 0, 1);
end

function ProximoGridIndicatorBorder:SetIndicator(frame, range, color, text,  value, maxValue, texture, start, duration, stack)
	frame:SetBackdropBorderColor(color.r, color.g, color.b, color.a);
end

function ProximoGridIndicatorBorder:SetBorderWidth(width, f)
	if (f == nil) then
		self:WithAllFrames(func);
	else
		func(f);
	end	
end


-- ICON

ProximoGridIndicatorIcon = ProximoGridIndicator.NewModule("icon", L["Center Icon"], {
	iconBorderWidth = 2;
	iconSize = 16;
}, {});

function ProximoGridIndicatorIcon:OnInitialize()
	self:RegisterEvent("PROXIMOGRID_TEXT_FONT_UPDATED")
end

ProximoGridIndicatorIcon.options = {
	iconBorderWidth = {
		type = "range",
		name = L["Icon Border Width"],
		desc = L["Icon border width."],
		get = function()
			return ProximoGridIndicatorIcon.db.iconBorderWidth;
		end,
		set = function(v)
			ProximoGridIndicatorIcon.db.iconBorderWidth = v;
			ProximoGridIndicatorIcon:SetIconSize(ProximoGridIndicatorIcon.db.iconSize, v);
		end,
		min = 2,
		max = 4,
		step = 1,
		order = 100,
	},
	iconSize = {
		type = "range",
		name = L["Icon Size"],
		desc = L["Icon size."],
		get = function()
			return ProximoGridIndicatorIcon.db.iconSize;
		end,
		set = function(v)
			ProximoGridIndicatorIcon.db.iconSize = v;
			ProximoGridIndicatorIcon:SetIconSize(v, ProximoGridIndicatorIcon.db.iconBorderWidth);
		end,
		min = 8,
		max = 40,
		step = 1,
		order = 120,
	},
};

function ProximoGridIndicatorIcon:CreateIndicator(frame)
	
	--- create icon background/border
	frame.IconBG = CreateFrame("Frame", nil, frame)
	frame.IconBG:SetFrameStrata("LOW")
	frame.IconBG:SetPoint("CENTER", frame, "CENTER")
	frame.IconBG:SetBackdrop( {
			-- bgFile = "Interface\\Addons\\Grid\\white16x16", tile = true, tileSize = 16,
			edgeFile = "Interface\\Addons\\ProximoGrid\\white16x16", edgeSize = 2,
			insets = {left = 2, right = 2, top = 2, bottom = 2},
		     })
	frame.IconBG:SetBackdropBorderColor(1,1,1,1)
	frame.IconBG:SetBackdropColor(0, 0, 0, 0)
	frame.IconBG:SetFrameLevel(7)
	frame.IconBG:Hide()

	-- create icon
	frame.Icon = frame.IconBG:CreateTexture("Icon", "OVERLAY")
	frame.Icon:SetPoint("CENTER", frame.IconBG, "CENTER")
	frame.Icon:SetTexCoord(0.05, 0.95, 0.05, 0.95)
	frame.Icon:SetTexture(1,1,1,1)
	frame.Icon:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark")

	-- create icon text
	frame.IconText = frame.IconBG:CreateFontString(nil, "OVERLAY")
	frame.IconText:SetAllPoints(frame.IconBG)
	frame.IconText:SetFontObject(GameFontHighlightSmall)
	--frame.IconText:SetFont(font, ProximoGrid.db.profile.unitFrame.fontSize)
	frame.IconText:SetJustifyH("CENTER")
	frame.IconText:SetJustifyV("CENTER")

	-- create icon cooldown
	frame.IconCD = CreateFrame("Cooldown", nil, frame.IconBG, "CooldownFrameTemplate")
	frame.IconCD:SetAllPoints(frame.Icon)
	frame.IconCD:Hide();
	frame.IconCD:SetScript("OnHide", function()
		frame.IconStackText:SetParent(frame.IconBG)
		frame.IconStackText:SetPoint("BOTTOMRIGHT", frame.IconBG, -ProximoGridIndicatorIcon.db.iconBorderWidth, ProximoGridIndicatorIcon.db.iconBorderWidth+2)
	end)
	

	-- create icon stack text
	frame.IconStackText = frame.IconBG:CreateFontString(nil, "OVERLAY")
	frame.IconStackText:SetPoint("BOTTOMRIGHT", frame.IconBG, 0, 4)
	frame.IconStackText:SetFontObject(GameFontHighlightSmall)
	--frame.IconStackText:SetFont(font, ProximoGrid.db.profile.unitFrame.fontSize, "OUTLINE")
	frame.IconStackText:SetJustifyH("RIGHT")
	frame.IconStackText:SetJustifyV("BOTTOM")
	local font = media:Fetch("font", ProximoGrid.db.profile.unitFrame.font);
	ProximoGridIndicatorIcon:SetFont(font, ProximoGrid.db.profile.unitFrame.fontSize, frame);
	ProximoGridIndicatorIcon:SetIconSize(ProximoGridIndicatorIcon.db.iconSize, ProximoGridIndicatorIcon.db.iconBorderWidth, frame);
end

function ProximoGridIndicatorIcon:ResetIndicator(frame)
	--frame.IconStackText:SetText("");
	frame.IconBG:Hide();
	frame.IconBG:SetBackdropBorderColor(0, 0, 0, 0);
end

function ProximoGridIndicatorIcon:SetIndicator(frame, range, color, text,  value, maxValue, texture, start, duration, stack)
	frame.Icon:SetTexture(texture);
	if (texture == "Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes") then
		local class = CLASS_BUTTONS[string.upper(text)];
		if class and class[1] and class[2] and class[3] and class[4] then
			frame.Icon:SetTexCoord(class[1], class[2], class[3], class[4])
		else
			frame.Icon:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark")
			frame.Icon:SetTexCoord(0,0,0,1,1,0,1,1)
		end
	elseif (texture == "Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Races") then
		local splitAt = string.find(text, ";");
		local raceString = string.sub(text, 1, splitAt-1);
		local sexSting = string.sub(text, splitAt+1);
		local race = ProximoGrid.raceIcons[raceString];
		local sexoffset = (sexSting == 3 and 0.5 or 0);

		if race and race[1] and race[2] and race[3] and race[4] then
			frame.Icon:SetTexCoord(race[1], race[2], race[3]+sexoffset, race[4]+sexoffset);
		else
			frame.Icon:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark");
			frame.Icon:SetTexCoord(0,0,0,1,1,0,1,1);
		end

	else
		frame.Icon:SetTexCoord(0.05, 0.95, 0.05, 0.95)
	end
	if (start ~= nil and start ~= 0) then
		frame.IconCD:Show();
		frame.IconStackText:SetParent(frame.IconCD)
		frame.IconStackText:SetPoint("BOTTOMRIGHT", frame.IconCD, 0, 2)
		frame.IconCD:SetCooldown(start, duration);
	else
		frame.IconCD:Hide();
	end
	
	if (stack ~= nil and stack ~= 0) then
		frame.IconStackText:SetText(stack);
	else
		frame.IconStackText:SetText("");
	end
	frame.IconBG:Show();
	frame.IconBG:SetBackdropBorderColor(color.r, color.g, color.b, color.a);
end

function ProximoGridIndicatorIcon:SetIconSize(size, borderWidth, f)
	local func = function (uf)
		local backdrop = uf.IconBG:GetBackdrop()
		backdrop.edgeSize = borderWidth
		backdrop.insets.left = borderWidth
		backdrop.insets.right = borderWidth
		backdrop.insets.top = borderWidth
		backdrop.insets.bottom = borderWidth
		local r, g, b, a = uf.IconBG:GetBackdropBorderColor()
		uf.IconBG:SetBackdrop(backdrop)
		uf.IconBG:SetBackdropBorderColor(r, g, b, a)
		uf.IconBG:SetWidth(size + borderWidth * 2)
		uf.IconBG:SetHeight(size + borderWidth * 2)
		uf.Icon:SetWidth(size)
		uf.Icon:SetHeight(size)
		if (uf.IconCD:IsShown()) then
			uf.IconStackText:SetPoint("BOTTOMRIGHT", uf.IconCD, 0, 2)
		else
			uf.IconStackText:SetPoint("BOTTOMRIGHT", uf.IconBG, -borderWidth, borderWidth+2);
		end
	end
	if (f == nil) then
		self:WithAllFrames(func);
	else
		func(f);
	end
end

function ProximoGridIndicatorIcon:SetFont(font, size, f)
	local func = function(uf)
		uf.IconStackText:SetFont(font, size, "OUTLINE");
		uf.IconText:SetFont(font, size, "OUTLINE");
	end
	if (f == nil) then
		self:WithAllFrames(func);
	else
		func(f);
	end	
end

function ProximoGridIndicatorIcon:PROXIMOGRID_TEXT_FONT_UPDATED(font, fontSize)
	ProximoGridIndicatorIcon:SetFont(font, fontSize);
end

-- FRAME ALPHA

ProximoGridIndicatorAlpha = ProximoGridIndicator.NewModule("alpha", L["Frame Alpha"], {}, {"statusNotTargetable"});
ProximoGridIndicatorAlpha.options = {};

function ProximoGridIndicatorAlpha:CreateIndicator(frame)
end

function ProximoGridIndicatorAlpha:ResetIndicator(frame)
	if (frame.unitID==nil or ProximoGrid.Opponents[frame.unitID] == nil) then
		frame:SetAlpha(0);
	else
		frame:SetAlpha(1);
	end
end

function ProximoGridIndicatorAlpha:SetIndicator(frame, range, color, text,  value, maxValue, texture, start, duration, stack)
	frame:SetAlpha(color.a);
end


-- CENTER TEXT

ProximoGridIndicatorText1 = ProximoGridIndicator.NewModule("text1", L["Center Text"], {
	enabled = true,
}, {
	"statusName",
	"statusHealth",
	"statusDeath"
});
ProximoGridIndicatorText1.options = {};
function ProximoGridIndicatorText1:OnInitialize()
	self:RegisterEvent("PROXIMOGRID_TEXT_LENGTH_UPDATED")
	self:RegisterEvent("PROXIMOGRID_TEXT_ORIENTATION_UPDATED")
	self:RegisterEvent("PROXIMOGRID_TEXT_FONT_UPDATED")
	self:RegisterEvent("PROXIMOGRID_TOGGLE_INDICATOR")
end

function ProximoGridIndicatorText1:CreateIndicator(frame)
	frame.Text = frame.Bar:CreateFontString(nil, "ARTWORK")
	frame.Text:SetFontObject(GameFontHighlightSmall)
	frame.Text:SetJustifyH("CENTER")
	frame.Text:SetJustifyV("CENTER")
	frame.Text:SetText(i);
	ProximoGridIndicatorText1:SetFont(media:Fetch("font", ProximoGrid.db.profile.unitFrame.font), ProximoGrid.db.profile.unitFrame.fontSize, frame);
	ProximoGridIndicatorText1:SetTextOrientation(ProximoGrid.db.profile.unitFrame.textOrientation, frame);
end

function ProximoGridIndicatorText1:ResetIndicator(frame)
	frame.Text:SetText("");
	frame.Text.textCache = "";
end

function ProximoGridIndicatorText1:SetIndicator(frame, range, color, text,  value, maxValue, texture, start, duration, stack)
	local number = tonumber(text);
	if (number ~= nil) then
		if(text:len()>ProximoGrid.db.profile.unitFrame.textLength) then
			if (number>1000000) then
				text = (math.floor(number/1000000+0.5)).."M";
			elseif (number>1000) then
				text = (math.floor(number/1000+0.5)).."K";
			end
		end
	end
	frame.Text:SetTextColor(color.r, color.g, color.b, color.a);
	frame.Text.textCache = text;
	frame.Text:SetText(text:sub(1, ProximoGrid.db.profile.unitFrame.textLength));
end

function ProximoGridIndicatorText1:SetTextOrientation(orientation, f)
	local func = function(uf)
		if (self.db.enabled) then
			local fs = uf.Text;
			local text2Enabled = ProximoGrid.UnitFrame:GetIndicator("text2").db.enabled;
			fs:ClearAllPoints();
			if (orientation == "VERTICAL") then
				fs:SetPoint("LEFT", uf, "LEFT", 1);
				fs:SetPoint("RIGHT", uf, "RIGHT", -1);
				if (text2Enabled) then
					fs:SetPoint("BOTTOM", uf, "CENTER");
				else
					fs:SetPoint("CENTER", uf, "CENTER");
				end
			else
				fs:SetPoint("CENTER", uf, "CENTER");
				fs:SetPoint("LEFT", uf, "LEFT");
				if (text2Enabled) then
					fs:SetPoint("RIGHT", uf, "CENTER");
				else
					fs:SetPoint("RIGHT", uf, "RIGHT");	
				end
			end
		end
	end
	if (f == nil) then
		self:WithAllFrames(func);
	else
		func(f);
	end
end

function ProximoGridIndicatorText1:SetFont(font, size, f)
	local func = function(uf)
		uf.Text:SetFont(font, size)
	end
	if (f == nil) then
		self:WithAllFrames(func);
	else
		func(f);
	end	
end

function ProximoGridIndicatorText1:SetTextLength(length, f)
	local func = function(uf)
		local text = uf.Text.textCache;
		if (text~=nil) then
			uf.Text:SetText(text:sub(1, length));
		end
	end
	if (f == nil) then
		self:WithAllFrames(func);
	else
		func(f);
	end
end

function ProximoGridIndicatorText1:PROXIMOGRID_TOGGLE_INDICATOR(indicator, enabled)
	if (indicator == "text2") then
		ProximoGridIndicatorText1:SetTextOrientation(ProximoGrid.db.profile.unitFrame.textOrientation);
	end
end

function ProximoGridIndicatorText1:PROXIMOGRID_TEXT_LENGTH_UPDATED(length)
	ProximoGridIndicatorText1:SetTextLength(length);
end

function ProximoGridIndicatorText1:PROXIMOGRID_TEXT_ORIENTATION_UPDATED(orientation)
	ProximoGridIndicatorText1:SetTextOrientation(orientation)
end


function ProximoGridIndicatorText1:PROXIMOGRID_TEXT_FONT_UPDATED(font, size)
	ProximoGridIndicatorText1:SetFont(font, size);
end
-- CENTER TEXT2

ProximoGridIndicatorText2 = ProximoGridIndicator.NewModule("text2", L["Center Text2"], {
	enabled = true,
}, {
	"statusDeath"
});
ProximoGridIndicatorText2.options = {
	showText2 = {
		type = "toggle",
		name = L["Enable Center Text 2 Indicator"],
		desc = L["Toggle Center Text 2 Indicator."],
		get = function()
			return ProximoGridIndicatorText2.db.enabled;
		end,
		set = function(v)
			ProximoGridIndicatorText2.db.enabled = v;
			ProximoGridIndicatorText2:WithAllFrames(function(uf)
				ProximoGridIndicatorText2:SetTextOrientation(ProximoGrid.db.profile.unitFrame.textOrientation, uf)
			end);
			ProximoGrid:EventDispatcher("PROXIMOGRID_TOGGLE_INDICATOR", "text2", v);
		end,
		order = 20,
	}
};
ProximoGridIndicatorText2.enabled = true;

function ProximoGridIndicatorText2:OnInitialize()
	self:RegisterEvent("PROXIMOGRID_TEXT_LENGTH_UPDATED")
	self:RegisterEvent("PROXIMOGRID_TEXT_ORIENTATION_UPDATED")
	self:RegisterEvent("PROXIMOGRID_TEXT_FONT_UPDATED")
	self:RegisterEvent("PROXIMOGRID_TOGGLE_INDICATOR")
end

function ProximoGridIndicatorText2:CreateIndicator(frame)
	frame.Text2 = frame.Bar:CreateFontString(nil, "ARTWORK")
	frame.Text2:SetFontObject(GameFontHighlightSmall)
	frame.Text2:SetJustifyH("CENTER")
	frame.Text2:SetJustifyV("CENTER")
	frame.Text2:SetText(i);
	frame.Text2:SetPoint("BOTTOM", frame, "CENTER")
	ProximoGridIndicatorText2:SetFont(media:Fetch("font", ProximoGrid.db.profile.unitFrame.font), ProximoGrid.db.profile.unitFrame.fontSize, frame);
	ProximoGridIndicatorText2:SetTextOrientation(ProximoGrid.db.profile.unitFrame.textOrientation, frame)
end

function ProximoGridIndicatorText2:ResetIndicator(frame)
	if (ProximoGridIndicatorText2.db.enabled) then
		frame.Text2:SetText("");
		frame.Text2.textCache = "";
	end
end

function ProximoGridIndicatorText2:SetIndicator(frame, range, color, text,  value, maxValue, texture, start, duration, stack)
	if (ProximoGridIndicatorText2.db.enabled) then
		local number = tonumber(text);
		if (number ~= nil) then
			if(text:len()>ProximoGrid.db.profile.unitFrame.textLength) then
				if (number>1000000) then
					text = (math.floor(number/1000000+0.5)).."M";
				elseif (number>1000) then
					text = (math.floor(number/1000+0.5)).."K";
				end
			end
		end
		frame.Text2:SetTextColor(color.r, color.g, color.b, color.a);
		frame.Text2:SetText(text:sub(1, ProximoGrid.db.profile.unitFrame.textLength));
		frame.Text2.textCache = text;
	end
end

function ProximoGridIndicatorText2:PROXIMOGRID_TEXT_LENGTH_UPDATED(length)
	ProximoGridIndicatorText2:SetTextLength(length)
end

function ProximoGridIndicatorText2:PROXIMOGRID_TEXT_ORIENTATION_UPDATED(orientation)
	ProximoGridIndicatorText2:SetTextOrientation(orientation)
end


function ProximoGridIndicatorText2:PROXIMOGRID_TEXT_FONT_UPDATED(font, size)
	ProximoGridIndicatorText2:SetFont(font, size);
end

function ProximoGridIndicatorText2:PROXIMOGRID_TOGGLE_INDICATOR(indicator, enabled)
	--ProximoGridIndicatorText2:SetTextOrientation(ProximoGrid.db.profile.unitFrame.textOrientation);
end

function ProximoGridIndicatorText2:ToggleText(enable, f)
	local func = function(uf)
		if (self.db.enabled) then
			fs:Show();
		else
			fs:Hide();
		end
	end;

	if (f == nil) then
		self:WithAllFrames(func);
	else
		func(f);
	end
end

function ProximoGridIndicatorText2:SetTextOrientation(orientation, f)
	local func = function(uf)
		local fs = uf.Text2;
		if (ProximoGridIndicatorText2.db.enabled) then
			fs:Show();
			fs:ClearAllPoints();
			if (orientation == "VERTICAL") then
				fs:SetPoint("LEFT", uf, "LEFT", 1);
				fs:SetPoint("RIGHT", uf, "RIGHT", -1);
				fs:SetPoint("TOP", uf, "CENTER");
			else
				fs:SetPoint("CENTER", uf, "CENTER");
				fs:SetPoint("LEFT", uf, "CENTER");
				fs:SetPoint("RIGHT", uf, "RIGHT");
			end
		else
			fs:Hide();
		end
	end;

	if (f == nil) then
		self:WithAllFrames(func);
	else
		func(f);
	end
end

function ProximoGridIndicatorText2:SetTextLength(length, f)
	local func = function(uf)
		local text = uf.Text2.textCache;
		if (text~=nil) then
			uf.Text2:SetText(text:sub(1, length));
		end
	end
	if (f == nil) then
		self:WithAllFrames(func);
	else
		func(f);
	end
end

function ProximoGridIndicatorText2:SetFont(font, size, f)
	local func = function(uf)
		uf.Text2:SetFont(font, size)
	end
	if (f == nil) then
		self:WithAllFrames(func);
	else
		func(f);
	end	
end

----------CORNERS---------
local function CreateCorner(frame, x)
	frame["Corner"..x] = CreateFrame("Frame", nil, frame);
	frame["Corner"..x]:SetBackdrop( {
		bgFile = "Interface\\Addons\\ProximoGrid\\white16x16", tile = true, tileSize = 16,
		edgeFile = "Interface\\Addons\\ProximoGrid\\white16x16", edgeSize = 1,
		insets = {left = 1, right = 1, top = 1, bottom = 1},
	});
	frame["Corner"..x]:SetBackdropBorderColor(0,0,0,1)
	frame["Corner"..x]:SetBackdropColor(1,1,1,1)
	frame["Corner"..x]:SetFrameStrata("LOW")
	frame["Corner"..x]:SetFrameLevel(10)
	if (x==1) then
		frame["Corner"..x]:SetPoint("TOPLEFT", frame, "TOPLEFT", 1, -1);
	elseif (x==2) then
		frame["Corner"..x]:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -1, -1);
	elseif (x==3) then
		frame["Corner"..x]:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", 1, 1);
	elseif (x==4) then
		frame["Corner"..x]:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -1, 1);
	end
	ProximoGrid.UnitFrame:GetIndicator("corner"..x):SetCornerSize(ProximoGrid.db.profile.unitFrame.cornerSize, frame);
	frame["Corner"..x]:Hide();
end

local function CornerSetIndicator(frame, range, color, text,  value, maxValue, texture, start, duration, stack, x)
	frame["Corner"..x]:SetBackdropColor(color.r, color.g, color.b, color.a);
	frame["Corner"..x]:Show();
end

local function CornerResetIndicator(frame, x)
	frame["Corner"..x]:SetBackdropColor(1, 1, 1, 1);
	frame["Corner"..x]:Hide();
end

local function CornerSetSize(size, x, f)
	local func = function(uf)
		uf["Corner"..x]:SetWidth(size);
		uf["Corner"..x]:SetHeight(size);
	end
	if (f == nil) then
		ProximoGrid.UnitFrame:GetIndicator("corner"..x):WithAllFrames(func);
	else
		func(f);
	end
end

local function CornerOnInitialize(x)
	ProximoGrid.UnitFrame:GetIndicator("corner"..x):RegisterEvent("PROXIMOGRID_UNIT_FRAME_CORNER_SIZE_UPDATED");
end

local function CornerPROXIMOGRID_UNIT_FRAME_CORNER_SIZE_UPDATED(size, x)
	ProximoGrid.UnitFrame:GetIndicator("corner"..x):SetCornerSize(size);
end


-- CORNER TOP LEFT
ProximoGridIndicatorCorner1 = ProximoGridIndicator.NewModule("corner1", L["Top Left Corner"], {}, {});
ProximoGridIndicatorCorner1.options = {};
ProximoGridIndicatorCorner1.x = 1;
function ProximoGridIndicatorCorner1:CreateIndicator(frame)
	CreateCorner(frame, self.x);
end
function ProximoGridIndicatorCorner1:ResetIndicator(frame)
	CornerResetIndicator(frame, self.x);
end
function ProximoGridIndicatorCorner1:SetIndicator(frame, range, color, text,  value, maxValue, texture, start, duration, stack)
	CornerSetIndicator(frame, range, color, text,  value, maxValue, texture, start, duration, stack, self.x);
end
function ProximoGridIndicatorCorner1:SetCornerSize(size, f)
	CornerSetSize(size, self.x, f)
end
function ProximoGridIndicatorCorner1:OnInitialize()
	CornerOnInitialize(self.x)
end
function ProximoGridIndicatorCorner1:PROXIMOGRID_UNIT_FRAME_CORNER_SIZE_UPDATED(size)
	ProximoGridIndicatorCorner1:SetCornerSize(size);
end
-- CORNER TOP RIGHT
ProximoGridIndicatorCorner2 = ProximoGridIndicator.NewModule("corner2", L["Top Right Corner"], {}, {});
ProximoGridIndicatorCorner2.options = {};
ProximoGridIndicatorCorner2.x = 2;
function ProximoGridIndicatorCorner2:CreateIndicator(frame)
	CreateCorner(frame, self.x);
end
function ProximoGridIndicatorCorner2:ResetIndicator(frame)
	CornerResetIndicator(frame, self.x);
end
function ProximoGridIndicatorCorner2:SetIndicator(frame, range, color, text,  value, maxValue, texture, start, duration, stack)
	CornerSetIndicator(frame, range, color, text,  value, maxValue, texture, start, duration, stack, self.x);
end
function ProximoGridIndicatorCorner2:SetCornerSize(size, f)
	CornerSetSize(size, self.x, f)
end
function ProximoGridIndicatorCorner2:OnInitialize()
	CornerOnInitialize(self.x)
end
function ProximoGridIndicatorCorner2:PROXIMOGRID_UNIT_FRAME_CORNER_SIZE_UPDATED(size)
	ProximoGridIndicatorCorner2:SetCornerSize(size);
end

-- CORNER BOTTOM LEFT
ProximoGridIndicatorCorner3 = ProximoGridIndicator.NewModule("corner3", L["Bottom Left Corner"], {}, {});
ProximoGridIndicatorCorner3.options = {};
ProximoGridIndicatorCorner3.x = 3;
function ProximoGridIndicatorCorner3:CreateIndicator(frame)
	CreateCorner(frame, self.x);
end
function ProximoGridIndicatorCorner3:ResetIndicator(frame)
	CornerResetIndicator(frame, self.x);
end
function ProximoGridIndicatorCorner3:SetIndicator(frame, range, color, text,  value, maxValue, texture, start, duration, stack)
	CornerSetIndicator(frame, range, color, text,  value, maxValue, texture, start, duration, stack, self.x);
end
function ProximoGridIndicatorCorner3:SetCornerSize(size, f)
	CornerSetSize(size, self.x, f)
end
function ProximoGridIndicatorCorner3:OnInitialize()
	CornerOnInitialize(self.x)
end
function ProximoGridIndicatorCorner3:PROXIMOGRID_UNIT_FRAME_CORNER_SIZE_UPDATED(size)
	ProximoGridIndicatorCorner3:SetCornerSize(size);
end

-- CORNER BOTTOM RIGHT
ProximoGridIndicatorCorner4 = ProximoGridIndicator.NewModule("corner4", L["Bottom Right Corner"], {}, {});
ProximoGridIndicatorCorner4.options = {};
ProximoGridIndicatorCorner4.x = 4;
function ProximoGridIndicatorCorner4:CreateIndicator(frame)
	CreateCorner(frame, self.x);
end
function ProximoGridIndicatorCorner4:ResetIndicator(frame)
	CornerResetIndicator(frame, self.x);
end
function ProximoGridIndicatorCorner4:SetIndicator(frame, range, color, text,  value, maxValue, texture, start, duration, stack)
	CornerSetIndicator(frame, range, color, text,  value, maxValue, texture, start, duration, stack, self.x);
end
function ProximoGridIndicatorCorner4:SetCornerSize(size, f)
	CornerSetSize(size, self.x, f)
end
function ProximoGridIndicatorCorner4:OnInitialize()
	CornerOnInitialize(self.x)
end
function ProximoGridIndicatorCorner4:PROXIMOGRID_UNIT_FRAME_CORNER_SIZE_UPDATED(size)
	ProximoGridIndicatorCorner4:SetCornerSize(size);
end


-- ADDITIONAL BAR
ProximoGridIndicatorBar2 = ProximoGridIndicator.NewModule("bar2", L["Extra Bar"], {
	anchor = "RIGHT",
	width = 10,
	enabled = false
}, {});
ProximoGridIndicatorBar2.options = {
	enabled = {
		type = "toggle",
		name = L["Enable Bar2"],
		desc = L["Shows an extra bar."],
		get = function()
			return ProximoGridIndicatorBar2.db.enabled;
		end,
		set = function(v)
			ProximoGridIndicatorBar2.db.enabled = v;
			ProximoGridIndicatorBar2:UpdateBar();
		end,
		order = 300,
	},
	width = {
		type = "range",
		name = L["Bar2 Width"],
		desc = L["If the anchor is LEFT or RIGH then it's the width else the hight of the extra bar."],
		get = function()
			return ProximoGridIndicatorBar2.db.width;
		end,
		set = function(v)
			ProximoGridIndicatorBar2.db.width = v;
			ProximoGridIndicatorBar2:UpdateBar();
		end,
		min = 2,
		max = 50,
		step = 1,
		order = 320,
	},
	anchor = {
		type = "text",
		name = L["Bar2 Anchor"],
		desc = L["Sets the bar's anchor relative to the unit frame."],
		get = function()
			return ProximoGridIndicatorBar2.db.anchor;
		end,
		set = function(v)
			ProximoGridIndicatorBar2.db.anchor = v;
			ProximoGridIndicatorBar2:UpdateBar();
		end,
		validate={["LEFT"] = L["LEFT"], ["BOTTOM"] = L["BOTTOM"], ["RIGHT"] = L["RIGHT"], ["TOP"] = L["TOP"]},
		order = 340,
	},

};
function ProximoGridIndicatorBar2:CreateIndicator(frame)
	frame.Bar2BG = frame:CreateTexture()
	frame.Bar2BG:SetPoint("CENTER", frame, "CENTER")
	-- create bar
	frame.Bar2 = CreateFrame("StatusBar", nil, frame)
	frame.Bar2:SetMinMaxValues(0,100)
	frame.Bar2:SetValue(100)
	--frame.Bar2:SetStatusBarColor(0,0,0,0.8)
	frame.Bar2:SetPoint("TOPLEFT", frame.Bar2BG, "TOPLEFT")
	frame.Bar2:SetPoint("BOTTOMRIGHT", frame.Bar2BG, "BOTTOMRIGHT")
	ProximoGridIndicatorBar2:SetBarColor(0, 0, 0, 0.8, frame);
	ProximoGridIndicatorBar2:UpdateBar(frame);
end
function ProximoGridIndicatorBar2:ResetIndicator(frame)
	if (ProximoGridIndicatorBar2.db.enabled) then
		ProximoGridIndicatorBar2:SetBarColor(0, 0, 0, 1, frame);
	end
end
function ProximoGridIndicatorBar2:SetIndicator(frame, range, color, text,  value, maxValue, texture, start, duration, stack)
	if (ProximoGridIndicatorBar2.db.enabled) then
		ProximoGridIndicatorBar2:SetBarColor(color.r, color.g, color.b, 0.8, frame);
		if (start == nil or start == 0) then
			frame.Bar2:SetMinMaxValues(0, maxValue);
			frame.Bar2:SetValue(value);
		else
			frame.Bar2:SetMinMaxValues(start, start+duration);
			frame.Bar2:SetValue(GetTime());
			frame.Bar2:SetScript("OnUpdate", self.OnUpdate);
		end
	end	
end

function ProximoGridIndicatorBar2:OnUpdate(timeElapsed)
	local min, max = self:GetMinMaxValues();
	if (max<=GetTime()) then
		self:SetScript("OnUpdate", nil);
		self:SetValue(min);
		return;
	end
	self:SetValue(GetTime());
end

function ProximoGridIndicatorBar2:UpdateBar(frame)
	local func = function(uf)
		if (not ProximoGridIndicatorBar2.db.enabled) then
			uf.Bar2BG:ClearAllPoints();
			uf.Bar2BG:Hide();
			uf.BarBG:SetPoint("TOP", uf, "TOP", 0, -1)
			uf.BarBG:SetPoint("BOTTOM", uf, "BOTTOM", 0, 1)
			uf.BarBG:SetPoint("LEFT", uf, "LEFT", 1, 0)
			uf.BarBG:SetPoint("RIGHT", uf, "RIGHT", -1, 0)
		else
			uf.Bar2BG:ClearAllPoints();
			uf.Bar2BG:Show();
			uf.BarBG:ClearAllPoints();
			local anchorArray = { 
				["TOP"] = "BOTTOM",
				["BOTTOM"] = "TOP",
				["LEFT"] = "RIGHT",
				["RIGHT"] = "LEFT"
			}
			local offsetX = {
				["TOP"] = 0,
				["BOTTOM"] = 0,
				["LEFT"] = 1,
				["RIGHT"] = -1
			}
			local offsetY = {
				["TOP"] = -1,
				["BOTTOM"] = 1,
				["LEFT"] = 0,
				["RIGHT"] = 0
			}
			for anchor, invertAnchor in pairs (anchorArray) do
				if (anchor == ProximoGridIndicatorBar2.db.anchor) then
					uf.BarBG:SetPoint(anchor, uf.Bar2, invertAnchor);
				else
					uf.Bar2BG:SetPoint(invertAnchor, uf, invertAnchor, offsetX[invertAnchor], offsetY[invertAnchor]);
					uf.BarBG:SetPoint(anchor,  uf, anchor, offsetX[anchor], offsetY[anchor]);	
				end
			end
			if (ProximoGridIndicatorBar2.db.anchor == "TOP" or ProximoGridIndicatorBar2.db.anchor == "BOTTOM") then
				uf.Bar2:SetOrientation("HORIZONTAL");
				uf.Bar2BG:SetHeight(ProximoGridIndicatorBar2.db.width);
			else
				uf.Bar2:SetOrientation("VERTICAL");
				uf.Bar2BG:SetWidth(ProximoGridIndicatorBar2.db.width);
			end
		end
	end
	if (frame==nil) then
		ProximoGridIndicatorBar2:WithAllFrames(func);
	else
		func(frame);
	end
end
function ProximoGridIndicatorBar2:OnInitialize()
	self:RegisterEvent("PROXIMOGRID_UNIT_FRAME_TEXTURE_UPDATED");
	self:RegisterEvent("PROXIMOGRID_UNIT_FRAME_INVERT_BAR_COLOR");
end

function ProximoGridIndicatorBar2:SetBarColor(r, g, b, a, frame)
	local func = function(uf)
		if (not ProximoGrid.db.profile.unitFrame.invertBarColor) then
			uf.Bar2:SetStatusBarColor(r, g, b, a)
			uf.Bar2BG:SetVertexColor(0, 0, 0, 0)
		else
			uf.Bar2:SetStatusBarColor(0, 0, 0, 0.8)
			uf.Bar2BG:SetVertexColor(r, g, b ,a)
		end
	end
	if (frame==nil) then
		ProximoGridIndicatorBar2:WithAllFrames(func);
	else
		func(frame);
	end
end

function ProximoGridIndicatorBar2:PROXIMOGRID_UNIT_FRAME_INVERT_BAR_COLOR(inverted)
	ProximoGridIndicatorBar2:WithAllFrames(function(uf)
		local r, g, b, a
		if (inverted) then
			r, g, b, a = uf.Bar2BG:GetVertexColor()
		else
			r, g, b, a = uf.Bar2:GetStatusBarColor()
		end
		ProximoGridIndicatorBar2:SetBarColor(r, g, b, a, uf);
	end);
end

function ProximoGridIndicatorBar2:PROXIMOGRID_UNIT_FRAME_TEXTURE_UPDATED(texture)
	ProximoGridIndicatorBar2:WithAllFrames(function(uf)
		uf.Bar2:SetStatusBarTexture(texture);
		uf.Bar2BG:SetTexture(texture);
	end);
end
